-- ------------------------------------------------------------------------------ --
--                           TradeSkillMaster_Vendoring                           --
--           http://www.curse.com/addons/wow/tradeskillmaster_vendoring           --
--                                                                                --
--             A TradeSkillMaster Addon (http://tradeskillmaster.com)             --
--    All Rights Reserved* - Detailed license information included with addon.    --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...)
local Buyback = TSM:NewModule("Buyback", "AceEvent-3.0", "AceHook-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Vendoring") -- loads the localization table

local private = { updateFunction = nil}

function Buyback:OnEnable()

end

function Buyback:CreateTab(parent)

	local frame = CreateFrame("Frame", nil, parent)
	frame:Hide()
	frame:SetAllPoints()
	frame:SetScript("OnShow", function() private.Show() end)
	
	local stContainer = CreateFrame("Frame", nil, frame)
	stContainer:SetPoint("TOPLEFT", 5, -45)
	stContainer:SetPoint("BOTTOMRIGHT", -5, 75)
	TSMAPI.Design:SetFrameColor(stContainer)
	
	local stCols = {
		{
			name = L["Item"],
			width = 0.6,
		},
		{
			name = L["Cost"],
			width = 0.4
		},
	}			
	
	local handlers = {
		OnClick = function(_, data, self, button)
			if not data then return end
											
			if button == "LeftButton" then							
				if IsShiftKeyDown() then
					BuybackItem(data.index)
					private.updateFunction = function() private:UpdateVendor() end
					TSMAPI:RegisterForBagChange(private.updateFunction)
				end										
			end
		end,
		OnEnter = function(_, data, self)
			if not data then return end
			local color = TSMAPI.Design:GetInlineColor("link")
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			TSMAPI:SafeTooltipLink(data.itemLink)
			GameTooltip:Show()
		end,
		OnLeave = function()
			GameTooltip:ClearLines()
			GameTooltip:Hide()
			BattlePetTooltip:Hide()
		end
	}

	local st = TSMAPI:CreateScrollingTable(stContainer, stCols, handlers)
	st:SetData({})
	frame.st = st	
	
	
	local btn = TSMAPI.GUI:CreateButton(frame, 18)	
	btn:SetPoint("BOTTOMLEFT", 5, 25)
	btn:SetWidth(70)
	btn:SetHeight(20)
	btn:SetText(L["Buyback"])
	btn:SetScript("OnClick", function() private:Buyback() end)
	frame.buybackBtn = btn		
	
	private.frame = frame
	return frame
end

function private:Buyback()
	if private.frame and private.frame.st then		
		local selection = private.frame.st:GetSelection()
		if selection then
			BuybackItem(selection)
			private:UpdateVendor()
		end	
	end
end

function private:Show()	
	private:UpdateVendor()
end

function private:UpdateVendor()
	local numBuybackItems = GetNumBuybackItems();
	
	local stData = {}	
		
	local buybackName, buybackTexture, buybackPrice, buybackQuantity, buybackNumAvailable, buybackIsUsable;	
	local itemCostTexture, itemCostValue, itemCostLink
	
	for index = 1, numBuybackItems do
		local buybackName, buybackTexture, buybackPrice, buybackQuantity, buybackNumAvailable, buybackIsUsable = GetBuybackItemInfo(index)
		local itemLink = GetBuybackItemLink(index)
		
		local displayCost = ""
		local displayCostSeparator = "" 
		local displayQuantity = ""
		
		if buybackPrice > 0 then
			displayCost = TSMAPI:FormatTextMoney(buybackPrice)
			displayCostSeparator = ", "
		end
		
		if buybackNumAvailable > -1 then
			displayQuantity = format("%d",buybackNumAvailable)
		end		
 					
		if itemLink then		
			tinsert(stData, { 
							cols = { 
								{ value = format("%s", itemLink) }, 
								{ value = displayCost }, 
								{ value = displayQuantity }
							}, 
							index = index,
							itemLink = itemLink,
						})
		end
	end		
	
	private.frame.st:SetData(stData)
end