-- ------------------------------------------------------------------------------ --
--                           TradeSkillMaster_Vendoring                           --
--           http://www.curse.com/addons/wow/tradeskillmaster_vendoring           --
--                                                                                --
--             A TradeSkillMaster Addon (http://tradeskillmaster.com)             --
--    All Rights Reserved* - Detailed license information included with addon.    --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...)
local Options = TSM:NewModule("Options", "AceEvent-3.0", "AceHook-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Vendoring") -- loads the localization table
local private = {ignoreST=nil, currentGroup=nil}

function Options:Load(parent, operation, group)
	Options.treeGroup = AceGUI:Create("TSMTreeGroup")
	Options.treeGroup:SetLayout("Fill")
	Options.treeGroup:SetCallback("OnGroupSelected", function(...) Options:SelectTree(...) end)
	Options.treeGroup:SetStatusTable(TSM.db.global.optionsTreeStatus)
	parent:AddChild(Options.treeGroup)

	Options:UpdateTree()
	if operation then
		if operation == "" then
			private.currentGroup = group
			Options.treeGroup:SelectByPath(3)
			private.currentGroup = nil
		else
			Options.treeGroup:SelectByPath(3, operation)
		end
	else
		Options.treeGroup:SelectByPath(1)
	end

	Options:HookScript(Options.treeGroup.frame, "OnHide", function()
		Options:UnhookAll()
		if private.ignoreST then private.ignoreST:Hide() end
	end)
end

function Options:UpdateTree()
	local operationTreeChildren = {}

	for name in pairs(TSM.operations) do
		tinsert(operationTreeChildren, { value = name, text = name })
	end
	
	sort(operationTreeChildren, function(a, b) return a.value < b.value end)

	Options.treeGroup:SetTree({ { value = 1, text = L["Options"] }, { value = 2, text = L["Ignore List"] }, { value = 3, text = L["Operations"], children = operationTreeChildren },  })
end

function Options:SelectTree(treeGroup, _, selection)
	treeGroup:ReleaseChildren()
	
	local major, minor = ("\001"):split(selection)
	major = tonumber(major)
		
	if major == 1 then
		Options:LoadGeneralSettings(treeGroup)
	elseif major == 2 then
		Options:DrawIgnoreSettings(treeGroup)
	elseif major == 3 then
		if minor then
			Options:DrawOperationSettings(treeGroup, minor)
		else	
			Options:DrawNewOperation(treeGroup)
		end
	end
end

function Options:DrawIgnoreSettings(container)

	local page = {
		{
			type = "SimpleGroup",
			fullHeight = true,
			layout = "Flow",
			children = {},
		},
	}

	TSMAPI:BuildPage(container, page)
	
	local stParent = container.children[1].frame
	
	if not private.ignoreST then
		local stCols = {
			{
				name = L["Ignored Item"],
				width = 1,
			}
		}
		
		local handlers = {
			OnEnter = function(_, data, self)
				if not data.itemString then return end
				GameTooltip:SetOwner(self, "ANCHOR_NONE")
				GameTooltip:SetPoint("BOTTOMLEFT", self, "TOPLEFT")
				GameTooltip:AddLine(L["Right click on this row to remove this item from the permanent ignore list."], 1, 1, 1, true)
				GameTooltip:Show()
			end,
			OnLeave = function()
				GameTooltip:ClearLines()
				GameTooltip:Hide()
			end,
			OnClick = function(_, data, _, button)
				if not data.itemString then return end
				if button == "RightButton" then
					TSM.db.global.ignore[data.itemString] = nil
					TSM:Printf(L["Removed %s from the permanent ignore list."], data.link)
					Options:UpdateIgnoreST()
				end
			end
		}

		private.ignoreST = TSMAPI:CreateScrollingTable(stParent, stCols, handlers)
	end
	
	private.ignoreST:Show()
	private.ignoreST:SetParent(stParent)
	private.ignoreST:SetAllPoints()
	Options:UpdateIgnoreST()
	
end

function Options:DrawNewOperation(container)
	local page = {
		{
			-- scroll frame to contain everything
			type = "ScrollFrame",
			layout = "List",
			children = {
				{
					type = "InlineGroup",
					layout = "flow",
					title = L["New Operation"],
					children = {
						{
							type = "Label",
							text = L["Vendoring operations contain settings for easy vendoring of items."],
							relativeWidth = 1,
						},
						{
							type = "EditBox",
							label = L["Operation Name"],
							relativeWidth = 0.8,
							callback = function(self, _, name)
								name = (name or ""):trim()
								if name == "" then return end
								if TSM.operations[name] then
									self:SetText("")
									return TSM:Printf(L["Error creating operation. Operation with name '%s' already exists."], name)
								end
								TSM.operations[name] = CopyTable(TSM.operationDefaults)
								Options:UpdateTree()
								Options.treeGroup:SelectByPath(3, name)
								TSMAPI:NewOperationCallback("Vendoring", private.currentGroup, name)
							end,
							tooltip = L["Give the new operation a name. A descriptive name will help you find this operation later."],
						},
					},
				},
			},
		},
	}
	TSMAPI:BuildPage(container, page)
end


function Options:DrawOperationSettings(container, operationName)
	local tg = AceGUI:Create("TSMTabGroup")
	tg:SetLayout("Fill")
	tg:SetFullHeight(true)
	tg:SetFullWidth(true)
	tg:SetTabs({{value=1, text=L["General"]}, {value=2, text=L["Relationships"]}, {value=3, text=L["Management"]}})
	tg:SetCallback("OnGroupSelected", function(self,_,value)
			tg:ReleaseChildren()
			TSMAPI:UpdateOperation("Vendoring", operationName)
			if value == 1 then
				Options:DrawOperationGeneral(self, operationName)
			elseif value == 2 then
				Options:DrawOperationRelationships(self, operationName)
			elseif value == 3 then
				TSMAPI:DrawOperationManagement(TSM, self, operationName)
			end
		end)
	container:AddChild(tg)
	tg:SelectTab(1)
end

function Options:DrawOperationGeneral(container, operationName)
	local operationSettings = TSM.operations[operationName]
	
	local page = {
		{
			-- scroll frame to contain everything
			type = "ScrollFrame",
			layout = "List",
			children = {
				{
					type = "InlineGroup",
					layout = "flow",
					title = L["Quick Sell Settings"],
					children = {						
						-- Temporarily disabling this until I get the code working						
--						{
--							type = "Slider",
--							label = L["Keep Quantity"],
--							relativeWidth = 0.5,
--							settingInfo = {operationSettings, "keepQty"},
--							disabled = operationSettings.relationships.keepQty,
--							min = 0,
--							max = 500,
--							step = 1,
--							tooltip = L["Vendoring will keep this number of items in the current player's bags."],
--						},
						{	
							type = "CheckBox",
							label = L["Do Not Sell"],
							settingInfo = {operationSettings, "qsDoNotSell"},
							disabled = operationSettings.relationships.qsDoNotSell,
							tooltip = L["If checked, a this item will not be sold to the vendor."],
						},			
												{
							type = "Slider",
							settingInfo = {operationSettings, "qsExpired"},
							label = L["Sell after expired auctions"],
							isPercent = false,
							min = 0,
							max = 100,
							step = 1,
							relativeWidth = 0.5,								
							disabled = operationSettings.relationships.qsExpired
						},
					},
				},
			},
		},
	}
	TSMAPI:BuildPage(container, page)
end

function Options:DrawOperationRelationships(container, operationName)
	local settingInfo = {
		{
			label = L["Quick Sell Settings"],
			{key="qsDoNotSell", label=L["Do Not Sell"]},
			{key="qsExpired", label=L["Sell after expired auctions"]},
			-- {key="keepQty", label=L["Keep Quantity"]},
		},
	}
	TSMAPI:ShowOperationRelationshipTab(TSM, container, TSM.operations[operationName], settingInfo)
end

function Options:LoadGeneralSettings(container)
	local page = {
		{
			type = "ScrollFrame",
			layout = "List",
			children = {
				{
					type = "InlineGroup",
					layout = "flow",
					title = L["General Settings"],
					relativeWidth = 1,
					children = {
						{
							type = "CheckBox",
							label = L["Make Vendoring Default Merchant Tab"],
							settingInfo = {TSM.db.global, "defaultMerchantTab"},
							tooltip = L["If checked, the Vendoring tab of merchant windows will be the default tab."],
						},
						{
							type = "Dropdown",
							label = L["Default Vendoring Page"],
							relativeWidth = 0.49,
							list = {L["Buy"], L["Buyback"], L["Sell"], L["Quick Sell"]},
							settingInfo = {TSM.db.global, "defaultPage"},
							tooltip = L["Specifies the default page that'll show when you select the TSM_Vendoring tab."],
						},		
					}
				},
				{
					type = "InlineGroup",
					layout = "flow",
					title = L["Quick Sell Settings"],
					relativeWidth = 1,
					children = {
						{
							type = "CheckBox",
							label = L["Hide Grouped Items"],
							settingInfo = { TSM.db.global, "qsHideGrouped" },
							tooltip = L["If checked, quicksell will hide any grouped items."],
						},
						{
							type = "Slider",
							label = L["Max number of items to sell when quickselling"],
							isPercent = false,
							min = 1,
							max = 100,
							step = 1,					
							settingInfo = { TSM.db.global, "qsNumToSell" },
							relativeWidth = 0.5,																	
						},					
						{
							type = "EditBox",
							label = L["Market Value"],
							settingInfo = { TSM.db.global, "qsMarketValue" },
							relativeWidth = 0.5,
							acceptCustom = true,
							tooltip = L["Formula for an item's market value"],
						},							
						{
							type = "EditBox",
							label = L["Default Max Market Value"],
							settingInfo = { TSM.db.global, "qsMaxMarketValue" },
							relativeWidth = 0.5,
							acceptCustom = true,
							tooltip = L["Maximum market value for an item to be vendored"],
						},	
						{
							type = "EditBox",
							label = L["Disenchant Value"],
							settingInfo = { TSM.db.global, "qsDisenchantValue" },
							relativeWidth = 0.5,
							acceptCustom = true,
							tooltip = L["Formula for an item's disenchant value"],
						},							
						{
							type = "EditBox",
							label = L["Default Max Disenchant Value"],
							settingInfo = { TSM.db.global, "qsMaxDisenchantValue" },
							relativeWidth = 0.5,
							acceptCustom = true,
							tooltip = L["Maximum disenchant value for an item to be vendored"],
						},		
						{
							type = "Slider",
							label = L["Sell after expired auctions"],
							isPercent = false,
							min = 0,
							max = 100,
							step = 1,					
							settingInfo = { TSM.db.global, "qsExpired" },
							relativeWidth = 0.5,																	
						},
					},
				},	
				{
					type = "InlineGroup",
					layout = "flow",
					title = L["Chat Message Options"],
					relativeWidth = 1,
					children = {
						{
							type = "CheckBox",
							label = L["Display Total Money Received"],
							settingInfo = {TSM.db.global, "displayMoneyCollected"},
							tooltip = L["If checked, the total amount of gold received will be shown at the end of automatically selling."],
						},
					},
				},
			},
		},
	}

	TSMAPI:BuildPage(container, page)
end

function Options:UpdateIgnoreST()
	if not private.ignoreST or not private.ignoreST:IsVisible() then return end
	local stData = {}
	for itemString in pairs(TSM.db.global.ignore) do
		local name, link = TSMAPI:GetSafeItemInfo(itemString)
		name = name or itemString
		link = link or itemString
		local row = {
			cols = {
				{
					value = link,
				},
			},
			name = name,
			link = link,
			itemString = itemString,
		}
		tinsert(stData, row)
	end
	sort(stData, function(a, b) return a.name < b.name end)
	private.ignoreST:SetData(stData)
end